<?php

if (!file_exists(dirname(__FILE__) . "/../config.php")) {
	die("Error: can't open configuration file: read the documentation about creating a new one!");
}

/* The dirname(__FILE__) chaos is needed to solve some issues
 * experienced with sourceforge hosting (i.e.: include_path empty
 * or variable or file mirroring between various hosts).
 *
 * Keep in mind that the order the file are included is VERY
 * important!
 *
 */
require(dirname(__FILE__) . "/../config.php");
require(dirname(__FILE__) . "/constants.php");
require(dirname(__FILE__) . "/core_functions.php");
require(dirname(__FILE__) . "/error.php");
require(dirname(__FILE__) . "/modules.php");
require(dirname(__FILE__) . "/log.php");
require(dirname(__FILE__) . "/session.php");
require(dirname(__FILE__) . "/mask.php");

global $CONFIG;

final class Core
{
	private $cExtList = array();
	private $cModNameList = array();
	private $cModList = array();

	/* External instances */
	private $cLogInstance;
	private $cSessionInstance;
	private $cModInstance;
	private $cStorageInstance;

	function __construct()
	{
		if (Config("debug")) {
			ini_set("error_reporting", E_ALL);
		}

		$this->_loadInstances();

		/* Setup the log manager */
		$this->addLog(DEBUG, "goWeb started at " .
				date("Y/m/d - H:i") . " on " .
				apache_get_version() . " (Zend " .
				zend_version() .
				"/PHP " . phpversion() . ")");

		/* look for extensions and modules in order to fill some
		 * control arrays
		 */
		if (ScanExtensions(&$this->cExtList) != ENOERROR) {
			ERROR("Can't get the extension list! Check the reading permissions!", $this);
		}
		$this->_loadBasicExtensions();
		if (ScanModules(&$this->cModNameList) != ENOERROR) {
			ERROR("Can't get the module list! Check the reading permissions!", $this);
		}

		/* storage is an apart case */
		$this->cStorageInstance = new Storage($this);

		/* Load the existing modules */
		$this->_loadModules();

		/* Now we can handle the MOD_PAGEINIT */
		$this->modCallEvent(MOD_PAGEINIT);
	}


	/* object overloading */

	function __get($var)
	{
		$this->addLog(SILENT, "CORE: Accessing to a variable here is not permitted");
	}

	function __set($var, $value)
	{
		$this->addLog(SILENT, "CORE: Can't set a variable in core. Use methods instead");
	}

	function __call($method, $arguments)
	{
		$this->addLog(IMPORTANT, "CORE: '$method' is a non-existent method in the core class");
	}

	/*************************************/


	public
	function loadExtension($extensionName)
	{
		if (!isset($extensionName)) {
			$this->addLog(SILENT, "CORE: Empty extension name passed to loadExtension");
			return ENOPARM;
		}

		/* NO: array_key_exists is so damn slow */
		if (!isset($this->cExtList[$extensionName])) {
			$this->addLog(WARNING, "CORE: Can't load specified extension");
			return ECANTLOAD;
		}

		if ($this->isExtensionLoaded($extensionName))
		{
			$this->addLog(SILENT, "CORE: Extension '$extensionName' already included. Skipping.");
		} else {
			if (!require(Config("site_path_fw") . "/extensions/$extensionName/$extensionName.php")) {
				$this->addLog(WARNING, "CORE: Can't load specified extension");
				return ENOFILEFOUND;
			}
			$this->_setExtensionLoaded($extensionName);

			$this->addLog(SILENT, "CORE: Extension '$extensionName' loaded");
		}

		return ENOERROR;
	}


	public
	function getExtensionList()
	{
		$retArray = array();
		foreach($this->cExtList as $extName => $extStatus)
		{
			if ($extStatus == TRUE) {
				$retArray[] = $extName;
			}
		}

		return $retArray;
	}


	public
	function isExtensionLoaded($extensionName)
	{
		if (	(isset($this->cExtList[$extensionName])) &&
			($this->cExtList[$extensionName] == TRUE)) {
			return TRUE;
		}

		return FALSE;
	}


	/************************/
	/* Log system interface */
	/************************/

	public
	function addLog($relevance, $msg)
	{
		return (($relevance == DEBUG) ? $this->cLogInstance->debug($msg) :
						$this->cLogInstance->addLogMsg($relevance, $msg));
	}


	public
	function getLog()
	{
		return $this->cLogInstance->getLog();
	}


	/*******************/
	/* Input interface */
	/*******************/


	public
	function getGet($getName, $raw = FALSE)
	{
		return ($raw ? $_GET[$getName] : $this->_preventSqlInjection($_GET[$getName]));
	}

	public
	function getGetArray($raw = FALSE)
	{
		$a = array();
		foreach($_GET as $key => $value)
		{
			$a[$key] = ($raw ? $_GET[$getName] : $this->_preventSqlInjection($_GET[$getName]));
		}
		return $a;
	}

	public
	function getPost($postName, $raw = FALSE)
	{
		return ($raw ? $_POST[$getName] : $this->_preventSqlInjection($_POST[$getName]));
	}

	public
	function getPostArray($raw = FALSE)
	{
		$a = array();
		foreach($_POST as $key => $value)
		{
			$a[$key] = ($raw ? $_POST[$getName] : $this->_preventSqlInjection($_POST[$getName]));
		}
		return $a;
	}

	/********************/
	/* Module interface */
	/********************/

	public
	function modRequire($instance, $extensionName)
	{
		$modName = getClassName($instance);
		$this->addLog(DEBUG, "MODULE: Module $modName requires the $extensionName extension");
		if (!$this->isExtensionLoaded($extensionName)) {
			$this->loadExtension($extensionName);
		}
	}


	public
	function modInit($modInstance, $modName, $modInfo)
	{
		$this->addLog(SILENT, "MODULE: Module $modName ($modInfo) registered and initialized");
		$this->cModInstance->init($modName, $modInfo, $modInstance);
	}


	public
	function modRegister($modInstance, $eventType, $funcName)
	{
		 $modName = getClassName($modInstance);
		 $this->cModInstance->register($modInstance, $eventType, $modName, $funcName);
	}


	public
	function modCallEvent($event, $moduleName = "")
	{
		return $this->cModInstance->callEventFunctions($event, $moduleName);
	}


	/***********/
	/* Storage */
	/***********/

	public
	function getStorageInstance()
	{
		return $this->cStorageInstance;
	}



	/*******************/
	/* private methods */
	/*******************/

	private
	function _loadModules()
	{
		foreach($this->cModNameList as $modName)
		{
			$this->cModList[$modName] = new $modName($this);
		}
	}


	private
	function _loadInstances()
	{
		$this->cLogInstance = new LogService($this);
		$this->cModInstance = new Modules($this);
		if (Config("session") == TRUE) {
			$this->cSessionInstance = new SessionService($this);
		}
	}


	private
	function _setExtensionLoaded($extensionName)
	{
		if (!isset($this->cExtList[$extensionName]))
			return ENOARRAYITEM;

		$this->cExtList[$extensionName] = TRUE;

		return ENOERROR;
	}


	private
	function _loadBasicExtensions()
	{
		$ext = array('storage');

		foreach($ext as $extension)
		{
			$this->loadExtension($extension);
		}
	}


	private
	function _preventSqlInjection($string, $raw = FALSE)
	{
		/* let see if we have GPC enabled */
		if (get_magic_quotes_gpc()) {
			return addslashes($this->_disableGpcMagicQuotes($string));
		} else return addslashes($string);
	}


	private
	function _disableGpcMagicQuotes($value)
	{
		return stripslashes($value);
	}
}

?>
