<?php

/* Template Engine
 *
 * This file contains the template engine class.
 * For documentation about using this class, let look at the doc/
 * directory.
 *
 */

interface iTemplate {
	public function set($varName, $varValue);
	public function setArray($varName, $arrayValue, $arrayEndString = "");
	public function output($fileName="", $outputAlways = FALSE);
}

class Template implements iTemplate
{
	private $cSiteInstance;
	private $cTemplatePath;
	private $cTemplateDir;

	private $cTemplateVar = array();
	private $cTemplateVarArray = array();

	private $cTemplateContent = array();


	public
	function __construct($siteInstance, $templateDir, $fileName)
	{
		if (func_num_args() != 3) {
			return ENOENOUGHPARMS;
		}

		/* we need this class to access at the logging system */
		$this->cSiteInstance = $siteInstance;

		/* check some params we need */
		if (	(Config("site_path") != ENOERROR) ||
			(Config("site_path_fw") != ENOERROR)) {
			ERROR("Some config value we need aren't available", $this->cSiteInstance);
		}

		$this->cTemplateDir = Config("site_path") . "/$templateDir/";
		$this->cTemplatePath = Config("site_path") . "/$templateDir/$fileName";
		if (!file_exists($this->cTemplatePath)) {
			ERROR("Can't open the template file (" .
				$this->cTemplatePath ."): check if it exists and its permissions",
				$this->cSiteInstance);
		}

		$this->cSiteInstance->addLog(DEBUG, "Opening template " . $this->cTemplatePath);

		/* setup some default var */
		$this->set("tpldir", "$templateDir/");
	}


	public
	function set($varName, $varValue)
	{
		if (func_num_args() != 2) {
			return ENOENOUGHPARMS;
		}

		/* we want it case insensitive */
		$varName = strtolower($varName);

		if (!isset($this->cTemplateVar[$varName])) {
			$this->cTemplateVar[$varName] = $varValue;
		} else {
			$this->cSiteInstance->addLog(WARNING, "Template variable \"$varName\" already used! Overwriting...");
			$this->cTemplateVar[$varName] = $varValue;
		}

		return ENOERROR;
	}


	public
	function setArray($varName, $arrayValue, $arrayEndString = "")
	{
		if ((func_num_args() < 2) || (func_num_args() > 3)) {
			return ENOENOUGHPARMS;
		}

		/* we want it case insensitive */
		$varName = strtolower($varName);

		if (strlen($arrayEndString) > 0) {
			addArrayValueEndString(&$arrayValue, $arrayEndString);
		}

		if (!isset($this->cTemplateVarArray[$varName])) {
			$this->cTemplateVarArray[$varName] = $arrayValue;
		} else {
			$this->cSiteInstance->addLog(WARNING, "Template variable \"$varName\" already used! Overwriting...");
			$this->cTemplateVarArray[$varName] = $arrayValue;
		}

		return ENOERROR;
	}


	public
	function output($fileName="", $outputAlways = FALSE)
	{	
		$this->cTemplateContent = @file($this->cTemplatePath);
		if (!isset($this->cTemplateContent)) {
			return ENOFILEFOUND;
		}

		$outputEnabled = (strlen($fileName) > 0) ? TRUE : FALSE;

		if ($outputEnabled) {
			$fout = fopen(Config("site_path") . "/$fileName", "w");

			if (!isset($fout)) {
				return ECANTOPEN;
			}
		}

		$debugStr = "Processing template ".$this->cTemplatePath." (file output ";
		if ($outputEnabled) {
			$debugStr .= "enabled on " . Config("site_path") . "/$fileName";
		} else {
			$debugStr .= "disabled";
		}
		$debugStr .= ")";	
		$this->cSiteInstance->addLog(DEBUG, $debugStr);

		$this->cTemplateContent = $this->_buildTemplateFile($this->cTemplateContent);

		foreach($this->cTemplateContent as $line)
		{
			if ($outputEnabled) {
				fwrite($fout, $line);

				if ($outputAlways == TRUE) {
					echo $line;
				}
			} else {
				echo $line;
			}
		}

		if ($outputEnabled == TRUE) {
			fclose($fout);
		}

		return ENOERROR;
	}


	private
	function _buildTemplateFile($tplContent)
	{
		/* TODO: this _must_ be recursive */
		foreach($tplContent as $rowNumber => $row)
		{
			/* for every row, look for template codes like {!...}, {#...} and others... */
			$i = 0;
			while ($i <= strlen($row))
			{
				$string = "";
				$errorOccurred = FALSE;
				if ((isset($row[$i])) && ($row[$i] == '{')) {
					$i++;
					while ((isset($row[$i])) && ($row[$i] != '}'))
					{
						if (($row[$i] == '\n') || ($row[$i] == ' '))
						{
							$errorOccurred = TRUE;
						}

						$string .= $row[$i];
						$i++;
					}
					$i--;
				}

				if ($errorOccurred) {
					$string = "";
				}

				if (strlen($string) > 0) {
					$command = $this->_getTemplateCommandFromString($string);

					/* remove the first string char since it should be a template command */
					$finalName = substr($string, 1);

					/*
					 * At this point we have:
					 * - $row 		the template's row analyzed
					 * - $command		a char indicating what template command it is
					 * - $finalName		the final template command
					 *
					 */
					$row = $this->_execTemplateCommand($row, $command, $finalName);
					$i--;
				}

				$i++;
			}
			$tplContent[$rowNumber] = $row;
		}

		return $tplContent;
	}


	private
	function _includeFile($fileName)
	{
		$tplFile = $this->cTemplateDir . "$fileName";
		$this->cSiteInstance->addLog(DEBUG, "Processing included template $tplFile");

		$fc = file($tplFile) OR
			ERROR("Can't open template included file " . $this->cTemplateDir . "$fileName", $this->cSiteInstance);

		$fc = $this->_buildTemplateFile($fc);
		return $this->_serializeArray($fc);
	}


	/* Returns the char indicating the template command */
	private
	function _getTemplateCommandFromString($string = "")
	{
		if (strlen($string) == 0) {
			return ENOPARM;
		}

		if ($this->_isTemplateChar($string[0])) {
			return $string[0];
		} else return EGENERIC;
	}


	private
	function _isTemplateChar($char)
	{
		switch ($char)
		{
			case '!':
			case '@':
			case '$':
			case '#':
			case '%':
				return TRUE;
				break;
			default:
				return FALSE;
				break;
		}
	}


	/*
	 * $row: the line to be analyzed
	 * $command: the char indicating which template action should be executed
	 * $value: the template command's value (i.e.: the name between the brackets)
	 *
	 */
	private
	function _execTemplateCommand($row, $command, $value)
	{
		switch($command)
		{
			case '$':
				/* replace variable */
				$row = $this->_replaceVarInLine($this->cTemplateVar, $row);
				break;
			case '!':
				/* include some file */
				$row = str_ireplace("{!$value}", $this->_includeFile($value), $row);
				break;
			case '@':
				/* replace with an array of lines */
				$row = $this->_replaceArrayInLine($this->cTemplateVarArray, $row);
				break;
			case '#':
				/* create a link to a section */
				$row = str_ireplace("{#$value}", $this->_replaceLink($value), $row);
				break;
			case '%':
				/* insert a module */
				$row = str_ireplace("{%$value}", $this->_replaceModule($value), $row);
				break;
			case EGENERIC:
			default:
				/* ignore it */
				break;
		}

		return $row;
	}


	private
	function _getToken($paramCount)
	{
		switch($paramCount)
		{
			case 0:
				$token = "";
				break;
			case 1:
				$token = '?';
				break;
			default:
				$token = '&amp;';
				break;
		}
		return $token;

	}


	private
	function _serializeArray($array)
	{
		if (!is_array($array)) {
			return EINVALIDPARMTYPE;
		}

		$serialized = "";
		foreach($array as $arrayElement)
		{
			$serialized .= $arrayElement;
		}
		return $serialized;
	}


	private
	function _replaceVarInLine($varArray, $string)
	{
		foreach($varArray as $varName => $varValue)
		{
			$origString = $string;
			$string = str_ireplace("{\$$varName}", $varValue, $string);
			if ($string != $origString) {
				$this->cSiteInstance->addLog(DEBUG, "Template var '$varName' substituted");
			}
		}

		return $string;
	}


	private
	function _replaceArrayInLine($varArray, $string)
	{
		foreach($varArray as $varName => $array)
		{
			$toReplace = "";
			foreach($array as $ar)
			{
				$toReplace .= "$ar";
			}

			$string =  str_ireplace("{@$varName}", $toReplace, $string);
		}

		return $string;
	}


	private
	function _replaceLink($linkTable)
	{
		$paramList = explode("|", $linkTable);
		$newvalue = "";
		$paramNum = 0;
		foreach($paramList as $param)
		{
			$prm = explode("=", $param);
			$newvalue .= $this->_getToken($paramNum) . $prm[0];
			if (isset($prm[1])) {
				$newvalue .= '=' . $prm[1];
			}
			$paramNum++;
		}
		return $newvalue;
	}

	private
	function _replaceModule($moduleName)
	{
		return $this->cSiteInstance->modCallEvent(MOD_OUTPUT, $moduleName);
	}
}


?>
